"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const composeParentsChain_1 = require("./composeParentsChain");
/**
 * Returns a notification object with
 * all origin object content set to
 * `undefined` (nested object included);
 *
 * @param obj
 * @param parents Optional parents
 */
function createChainFromObject(obj, parents = [], isUndefined = false) {
    if (!obj) {
        return undefined;
    }
    const chains = {};
    const objKeys = Object.keys(Object.getOwnPropertyDescriptors(obj));
    if (!objKeys.length) {
        return undefined;
    }
    for (let i = objKeys.length, prop; prop = objKeys[--i];) {
        const parentChains = composeParentsChain_1.composeParentsChains(prop, parents);
        if (typeof obj[prop] === "object") {
            Object.assign(chains, createChainFromObject(obj[prop], parentChains, isUndefined));
        }
        const newValue = isUndefined ? undefined : obj[prop];
        parentChains.forEach(c => chains[c] = newValue);
    }
    return chains;
}
exports.createChainFromObject = createChainFromObject;
